@@ -327,7 +327,8 @@
              [else
               (set! current-file file)
               (set! current-line line)
-              (format port "\n#line ~a ~s\n" line file)])]
+              (when (cise-emit-source-line)
+                (format port "\n#line ~a ~s\n" line file))])]
       ['|#reset-line| ; reset source info
        (set! current-file #f) (set! current-line 0)]
       [(x . y) (rec x) (rec y)]
@@ -442,8 +443,14 @@
 ;;
 
 ;; (define-cfn <name> (<arg> ...) [<rettype> [<qualifier> ...]] <body>)
+;; (declare-cfn <name> (<arg> ...) [<rettype> [<qualifier> ...]])
 
 (define-cise-macro (define-cfn form env)
+  (expand-cfn form env))
+(define-cise-macro (declare-cfn form env)
+  (expand-cfn form env))
+
+(define (expand-cfn form env)
   (define (gen-args args env)
     (let1 eenv (expr-env env)
       ($ intersperse ","
@@ -453,7 +460,8 @@
     (intersperse " "
                  (map (^[qual] (ecase qual
                                  [(:static) "static"]
-                                 [(:inline) "inline"]))
+                                 [(:inline) "inline"]
+                                 [(:extern) "extern"]))
                       (reverse quals))))
 
   (define (gen-cfn name quals args rettype body)
@@ -488,9 +496,19 @@
       [((? keyword? z) . body)
        (errorf "Invalid qualifier in define-cfn ~s: ~s" name z)]
       [_
-       (when (memq :static quals)
-         (record-static name quals args ret-type))
-       (gen-cfn name quals args ret-type body)]))
+       (case (car form)
+         [(define-cfn)
+          (when (memq :static quals)
+            (record-static name quals args ret-type))
+          (gen-cfn name quals args ret-type body)]
+         [(declare-cfn)
+          (unless (null? body)
+            (errorf "declare-cfn ~s must not have a body" name))
+          (when (or (memq :static quals) (memq :inline quals))
+            (errorf "declare-cfn ~s cannot have qualifier(s)" name))
+          (record-static name '(:extern) args ret-type)
+          ;; no function implementation
+          '()])]))
 
   (ensure-toplevel-ctx form env)
   (match form
@@ -501,6 +519,80 @@
     [(_ name (args ...) . body)
      (check-quals name '() (canonicalize-argdecl args) 'ScmObj body)]))
 
+;;------------------------------------------------------------
+;; Global variable definition and typedef
+;;
+
+;; (define-cvar <name> [::<type>] [<qualifiers>...] [<init>])
+;; (declare-cvar <name> [::<type>])
+;; (define-ctype <name> [::<type>])
+
+(define-cise-macro (define-cvar form env)
+  (expand-cvar form env #t))
+(define-cise-macro (declare-cvar form env)
+  (expand-cvar form env #f))
+(define-cise-macro (define-ctype form env)
+  (expand-cvar form env #f))
+
+(define (expand-cvar form env toplevel-only?)
+  (define (gen-qualifiers quals)
+    (intersperse " "
+                 (map (^[qual] (ecase qual
+                                      [(:static) "static"]
+                                      [(:extern) "extern"]
+                                      [(:typedef) "typedef"]))
+                      (reverse quals))))
+
+  (define (gen-cvar var type quals has-init? init)
+    `(,@(gen-qualifiers quals) " "
+      ,(cise-render-typed-var type var env)
+      ,@(cond-list [has-init? `(" = ",(render-rec init (expr-env env)))])
+      ";"))
+
+  (define (check-quals var type quals init-and-quals)
+    (match init-and-quals
+      [(':static . init-and-quals)
+       (check-quals var type `(:static ,@quals) init-and-quals)]
+      [((? keyword? z) . body)
+       (errorf "Invalid qualifier in define-cvar ~s: ~s" var z)]
+      [()
+       (case (car form)
+         [(define-cvar) (gen-cvar var type quals #f #f)]
+         [(declare-cvar)
+          (unless (null? quals)
+            (errorf "declare-cvar ~s cannot have qualifier(s)" var))
+          (gen-cvar var type '(:extern) #f #f)]
+         [(define-ctype)
+          (unless (null? quals)
+            (errorf "define-ctype ~s cannot have qualifier(s)" var))
+          (gen-cvar var type '(:typedef) #f #f)])]
+      [(init)
+       (if (eq? (car form) 'define-cvar)
+         (gen-cvar var type quals #t init)  
+         (errorf "declare-cvar ~s cannot have initializer" var))]
+      [else
+       (errorf "Invalid syntax in ~s ~s: ~s"
+               (car form) var init-and-quals)]))
+
+  ;; We allow define-cvar only on toplevel, but declare-cvar and
+  ;; define-ctype can appear in stmts.
+  (if toplevel-only?
+    (ensure-toplevel-ctx form env)
+    (ensure-stmt-or-toplevel-ctx form env))
+
+  (let* ([canon (car (canonicalize-vardecl (list (cdr form))))]
+         [var (car canon)]
+         [spec (cdr canon)])
+    (receive (type init-and-quals)
+        (match spec
+          [()         (values 'ScmObj '())]
+          [('::)      (errorf "invalid variable decl in ~s: (~s ~s)" 
+                              (car form) var spec)]
+          [(':: type) (values type '())]
+          [(':: type . init-and-quals) (values type init-and-quals)]
+          [else (values 'ScmObj spec)])
+      (check-quals var type '() init-and-quals))))
+
 ;;------------------------------------------------------------
 ;; CPS transformation
 ;;
@@ -796,7 +888,7 @@
        `(let* ((,var :: int 0) (,n :: int ,expr))
           (for [() (< ,var ,n) (post++ ,var)] ,@body))])))
 
-;; [cise stmt] return EXPR
+;; [cise stmt] return [EXPR]
 ;;   Return statement.
 ;;   NB: While processing cproc body in stubs, return macro is overwritten
 ;;   to handle multiple value returns.  See cgen-stub-cise-ambient
@@ -831,24 +923,48 @@
 ;; Preprocessor directives
 ;;
 
-;; [cise stmt] .if STRING STMT [STMT]
+;; [cise toplevel/stmt] .if STRING STMT [STMT]
 ;;   c preprocessor directive
 (define-cise-macro (.if form env)
   (ensure-stmt-or-toplevel-ctx form env)
   (match form
     [(_ condition stmt1)
      `("\n" |#reset-line|               ;make sure we start from the fresh line
-       "#if " ,(x->string condition) "\n" |#reset-line|
+       "#if " ,(cpp-condition->string condition) "\n" |#reset-line|
        ,(render-rec stmt1 env) "\n"
-       "#endif /* " ,(x->string condition) " */\n" |#reset-line|)]
+       "#endif /* " ,(cpp-condition->string condition) " */\n" |#reset-line|)]
     [(_ condition stmt1 stmt2)
      `("\n" |#reset-line|               ;make sure we start from the fresh line
-       "#if " ,(x->string condition) "\n" |#reset-line|
+       "#if " ,(cpp-condition->string condition) "\n" |#reset-line|
        ,(render-rec stmt1 env) "\n"
-       "#else  /* !",(x->string condition) " */\n" |#reset-line|
+       "#else /* !",(cpp-condition->string condition) " */\n" |#reset-line|
        ,(render-rec stmt2 env) "\n"
-       "#endif /* " ,(x->string condition) " */\n" |#reset-line|)]))
+       "#endif /* " ,(cpp-condition->string condition) " */\n" |#reset-line|)]))
 
+;; [cise toplevel/stmt] .when STRING STMT [STMT]
+;;   c preprocessor directive
+(define-cise-macro (.when form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ condition stmt ...)
+     `("\n" |#reset-line|               ;make sure we start from the fresh line
+       "#if " ,(cpp-condition->string condition) "\n" |#reset-line|
+       ,(intersperse "\n" (map (cut render-rec <> env) stmt)) "\n"
+       "#endif /* " ,(cpp-condition->string condition) " */\n" |#reset-line|)]))
+
+;; [cise toplevel/stmt] .unless STRING STMT [STMT]
+;;   c preprocessor directive
+(define-cise-macro (.unless form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ condition stmt ...)
+     `("\n" |#reset-line|               ;make sure we start from the fresh line
+       "#if !(" ,(cpp-condition->string condition) ")\n" |#reset-line|
+       ,(intersperse "\n" (map (cut render-rec <> env) stmt)) "\n"
+       "#endif /* ! " ,(cpp-condition->string condition) " */\n" |#reset-line|)]))
+
+;; [cise toplevel/stmt] .cond CLAUSE [CLAUSE]
+;;   c preprocessor if/elif/endif chain directive
 (define-cise-macro (.cond form env)
   (ensure-stmt-or-toplevel-ctx form env)
   (match form
@@ -856,18 +972,42 @@
      `("\n#if 0 /*dummy*/\n" |#reset-line|
        ,@(fold-right (lambda (c ss seed)
                        `(,(cond [(eq? c 'else) '("#else")]
-                                [else `("#elif " ,(x->string c))])
+                                [else `("#elif " ,(cpp-condition->string c))])
                          "\n" |#reset-line|
                          ,(map (cut render-rec <> env) ss) "\n"
                          ,@seed))
                      '("#endif\n" |#reset-line|)
                      condition stmts))]))
 
+;; [cise toplevel/stmt] .define NAME [EXPR]
+;; [cise toplevel/stmt] .define NAME (ARGS...) EXPR
+;;   c preprocessor define directive
+
+;; Note that "#define abc(a,b)" (i.e. no EXPR) cannot be generated
+;; because it's ambiguous with "#define abc a(b)".
+(define-cise-macro (.define form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ name) `("#define " ,(x->string name) "\n" |#reset-line|)]
+    [(_ name (args ...) expr) `("#define " ,(x->string name)
+                                "(" ,(intersperse "," (map x->string args)) ")"
+                                " (" ,(parameterize ([cise-emit-source-line #f])
+                                        (render-rec expr (expr-env env))) ")"
+                                "\n" |#reset-line|)]
+    [(_ name expr) `("#define " ,(x->string name)
+                     " (" ,(parameterize ([cise-emit-source-line #f])
+                             (render-rec expr (expr-env env))) ")"
+                     "\n" |#reset-line|)]))
+
+;; [cise toplevel/stmt] .undef NAME
+;;   c preprocessor undefine directive
 (define-cise-macro (.undef form env)
   (ensure-stmt-or-toplevel-ctx form env)
   (match form
     [(_ name) `("#undef " ,(x->string name) "\n" |#reset-line|)]))
 
+;; [cise toplevel/stmt] .include PATH
+;;   c preprocessor include directive
 (define-cise-macro (.include form env)
   (ensure-stmt-or-toplevel-ctx form env)
   (match form
@@ -1123,6 +1263,8 @@
 (define-cise-macro (.type form env)
   (match form
     [(_ typenames ...)
+     (when (null? typenames)
+       (errorf "empty .type form is not allowed: ~s" form))
      `(,(cise-render-typed-var typenames "" env))]))
 
 ;;------------------------------------------------------------
@@ -1183,7 +1325,7 @@
 ;; appear in the operator position of a valid expression.
 (define (type-decl-initial? sym)
   (or (memq sym '(const class enum struct volatile unsigned long
-                  char short int float double .array))
+                  char short int float double .array .struct .union .function))
       (and (symbol? sym)
            (#/.[*&]$/ (symbol->string sym)))))
 
@@ -1191,7 +1333,6 @@
   (or (memq sym '(* &))
       (type-decl-initial? sym)))
 
-
 (define (cise-render-typed-var typespec var env)
   (match typespec
     [('.array spec (dim ...))
@@ -1199,9 +1340,55 @@
        ,@(map (^.['* "[]"]
                  [x `("[" ,(render-rec x (expr-env env)) "]")])
               dim))]
-    [(x ...) `(,(intersperse " " (map x->string x))
-               " " ,(cise-render-identifier var))]
-    [x       `(,(x->string x) " " ,(cise-render-identifier var))]))
+    [('.struct (fields ...) . rest)
+     (render-struct-or-union "struct" #f fields rest var env)]
+    [('.struct tag (fields ...) . rest)
+     (render-struct-or-union "struct" tag fields rest var env)]
+    [('.struct tag . rest)
+     (render-struct-or-union "struct" tag #f rest var env)]
+    [('.union (fields ...) . rest)
+     (render-struct-or-union "union" #f fields rest var env)]
+    [('.union tag (fields ...) . rest)
+     (render-struct-or-union "union" tag fields rest var env)]
+    [('.union tag . rest)
+     (render-struct-or-union "union" tag #f rest var env)]
+    [('.function (args ...) rettype . rest)
+     (let1 rt (let1 vv (canonicalize-vardecl `(_ ,rettype))
+                 (unless (null? (cdr vv))
+                  (errorf "Invalid return type in ~s" typespec))
+                (caddar vv))
+       `(,(cise-render-typed-var rt "" env)
+         "("
+         ,(if (null? rest)
+            (cise-render-identifier var)
+            (cise-render-typed-var rest var env))
+         ")"
+         "("
+         ,@($ intersperse ", "
+              $ map (^.[(arg ':: type) (cise-render-typed-var type arg env)])
+              $ canonicalize-vardecl args)
+         ")"))]
+    [(x)
+     `(,(x->string x) " " ,(cise-render-identifier var))]
+    [(x xs ...) 
+     `(,(x->string x) " " ,(cise-render-typed-var xs var env))]
+    [x
+     `(,(x->string x) " " ,(cise-render-identifier var))]))
+
+(define (render-struct-or-union struct/union tag fields rest var env)
+  `(,struct/union
+    ,@(cond-list [tag `(" " ,tag)]
+                 [fields 
+                  `(" { "
+                    ,@(map (^.[(member ':: type)
+                               `(,(cise-render-typed-var type member env)
+                                 "; ")])
+                           (canonicalize-vardecl fields))
+                    "} ")]
+                 [(not fields) '(" ")])
+    ,(if (null? rest)
+       (cise-render-identifier var)
+       (cise-render-typed-var rest var env))))
 
 (define (cise-render-identifier sym)
   (cgen-safe-name-friendly (x->string sym)))
@@ -1223,7 +1410,7 @@
         [else (cons elt seed)])]
      [else (cons elt seed)]))
 
-  (define (err decl) (error "invlaid variable declaration:" decl))
+  (define (err decl) (error "invalid variable declaration:" decl))
 
   (define (scan in r)
     (match in
