@@ -292,6 +292,14 @@
   `(make <form-parser> :name ',name :args ',args
          :handler (^ (&whole ,@args) ,@body)))
 
+(define-syntax export-toplevel-cise-form
+  (syntax-rules ()
+    [(_ name)
+     (define-form-parser name args
+       (parameterize ([cise-ambient (cise-ambient-copy (cise-ambient) '())])
+         (cgen-body (cise-render-to-string `(name ,@args) 'toplevel))
+         (cgen-decl (cise-ambient-decl-strings (cise-ambient)))))]))
+
 (define-method invoke ((self <form-parser>) form)
   (define (badform)
     (errorf <cgen-stub-error> "stub: malformed ~a: ~s" (~ self'name) form))
@@ -326,10 +334,18 @@
 (define-form-parser define-cise-expr args
   (eval `(define-cise-expr ,@args) (current-module)))
 
-(define-form-parser define-cfn args
-  (parameterize ([cise-ambient (cise-ambient-copy (cise-ambient) '())])
-    (cgen-body (cise-render-to-string `(define-cfn ,@args) 'toplevel))
-    (cgen-decl (cise-ambient-decl-strings (cise-ambient)))))
+;; toplevel CiSE forms
+(export-toplevel-cise-form declare-cfn)
+(export-toplevel-cise-form declare-cvar)
+(export-toplevel-cise-form define-cfn)
+(export-toplevel-cise-form define-ctype)
+(export-toplevel-cise-form define-cvar)
+
+; CiSE forms .if, .cond, .when and .unless are not exported because
+; there are (or will be) stub replacements of the same name.
+(export-toplevel-cise-form .define)
+(export-toplevel-cise-form .include)
+(export-toplevel-cise-form .undef)
 
 ;; extra check of valid clauses
 (define (check-clauses directive name clauses valid-keys)
@@ -1792,7 +1808,7 @@
 ;;              subclass of <foreign-pointer> to represent this C-ptr type.
 ;;  c-type    : Name of the actual C type we wrap.
 ;;  c-name    : A C variable name (must have type ScmClass *).  If :private
-;;              qualifier is given, static definition of the varaible is
+;;              qualifier is given, the static definition of the variable is
 ;;              generated; otherwise, the definition & declaration must
 ;;              be provided elsewhere.  In initialization code, an instance
 ;;              of a class (the same one bound to scm-name in the Scheme
@@ -1898,6 +1914,7 @@
 ;; Main parsers
 ;;
 
+;; deprecated
 (define-form-parser if (test then . maybe-else)
   (cgen-with-cpp-condition test
     (cgen-stub-parse-form then))
@@ -1905,13 +1922,29 @@
     (cgen-with-cpp-condition `(not ,test)
       (cgen-stub-parse-form (car maybe-else)))))
 
+(define-form-parser .if (test then . maybe-else)
+  (cgen-with-cpp-condition test
+    (cgen-stub-parse-form then))
+  (unless (null? maybe-else)
+    (cgen-with-cpp-condition `(not ,test)
+      (cgen-stub-parse-form (car maybe-else)))))
+
 (define-form-parser begin forms
   (for-each cgen-stub-parse-form forms))
 
+;; deprecated
 (define-form-parser when (test . forms)
   (cgen-with-cpp-condition test
     (for-each cgen-stub-parse-form forms)))
 
+(define-form-parser .when (test . forms)
+  (cgen-with-cpp-condition test
+    (for-each cgen-stub-parse-form forms)))
+
+(define-form-parser .unless (test . forms)
+  (cgen-with-cpp-condition `(not ,test)
+    (for-each cgen-stub-parse-form forms)))
+
 (define-form-parser include (file)
   (unless (file-exists? file)
     ;; TODO: search path
