@@ -88,9 +88,11 @@
 (define (entry-name e)
   (if-let1 m (#/ <\d+>$/ e) (rxmatch-before m) e))
   
-(define *pager* (or (sys-getenv "PAGER")
-                    (find-file-in-paths "less")
-                    (find-file-in-paths "more")))
+(define *pager* (or (and-let1 s (sys-getenv "PAGER")
+                      (shell-tokenize-string s))
+                    (and-let1 cmd (or (find-file-in-paths "less")
+                                      (find-file-in-paths "more"))
+                      (list cmd))))
 
 (define (viewer-pager s)
   (let1 p (run-process *pager* :input :pipe)
