@@ -202,7 +202,7 @@
   (require "srfi-43")
 
   (import r7rs.aux)
-  (export * + - ... / < <= = => > >= abs and append apply assoc assq
+  (export * + - ... / < <= = => > >= _ abs and append apply assoc assq
           assv begin binary-port?  boolean=?  boolean?  bytevector
           bytevector-append bytevector-copy bytevector-copy! bytevector-length
           bytevector-u8-ref bytevector-u8-set!  bytevector?  caar cadr
@@ -309,15 +309,6 @@
   ;; vector-copy vector-copy! vector-append vector-fill!
 
   ;; 6.9 Bytevectors
-  (define-inline bytevector         u8vector)
-  (define-inline bytevector?        u8vector?)
-  (define-inline make-bytevector    make-u8vector)
-  (define-inline bytevector-length  u8vector-length)
-  (define-inline bytevector-u8-ref  u8vector-ref)
-  (define-inline bytevector-u8-set! u8vector-set!)
-  (define-inline bytevector-copy    u8vector-copy)
-  (define-inline bytevector-copy!   u8vector-copy!)
-  (define-inline bytevector-append  u8vector-append)
   (define+ utf8->string  gauche.unicode)
   (define+ string->utf8  gauche.unicode)
 
@@ -385,7 +376,7 @@
                           ENOSPC ENOTBLK ENOTDIR ENOTEMPTY ENXIO
                           EPERM EPIPE ESPIPE ESTALE ETXTBSY EXDEV)))))
 
-  ;; 6.12 Enviornments and evaluation
+  ;; 6.12 Environments and evaluation
   ;; scheme-report-environment null-environment
 
   ;; 6.13 Input and output
@@ -597,6 +588,8 @@
           symbol->string symbol? tan truncate values vector vector->list
           vector-fill! vector-length vector-ref vector-set! vector?
           with-input-from-file with-output-to-file write write-char zero?
+          ;; R7RS Errata #22
+          syntax-rules else ... => _
           )
   (provide "scheme/r5rs")
   )
